% ---------------------------------------------------------------------
% Publication:	Implied Correlation Dynamics
% ---------------------------------------------------------------------
% Quantlet:		DSFM_FPCA_ICS
% ---------------------------------------------------------------------
% Description:	DSFM_FPCA_ICS estimates DSFM model for implied
%               correlation surfaces (ICS) with basis functions
%               estimated in FPCA framework, the program uses
%               functions and inputs listed below
% ---------------------------------------------------------------------
% Functions:	fpca.m
%               loaddata.m
%               mu_est.m
%               mugrid.m
%               phi_est.m
%               phigrid.m
%               plotsurf.m
%               plotsurf_basis.m
%               plotsurf_data.m
%               plotsurf_interp.m
%               plotsurf_movie.m
%               z_est.m
% ---------------------------------------------------------------------
% Inputs:		SP500_IC_200905-200910.txt
% ---------------------------------------------------------------------
% Author:		Elena Silyakova, 20110511
% ---------------------------------------------------------------------

%%
clear all
clc
%% Loading and transforming data
[Date, Tau, Money, Rho]=loaddata();
%% Setting grid for nonparametric estimation
[moneygrid, taugrid, dim, Tau, Money, Rho,Date]=mugrid(Tau, Money, Rho, Date); % Grid for mean function
%% Local linear MEAN function estimation with bandwidth h_mu_opt
h_mu_opt=[0.0578,0.0155];
mu_u=mu_est(Money,Tau,Rho,h_mu_opt,moneygrid,taugrid); % Mean function via Nadaraya-Watson - mu_u(mu_v)
%% Plotting MEAN function
plotsurf(moneygrid,taugrid,mu_u) % Plot only Surface
plotsurf_interp(moneygrid,taugrid,mu_u) % Plot Interpolated surface
plotsurf_data(Money, Tau, Rho,moneygrid,taugrid,mu_u) % Plot the raw data
plotsurf_movie(Money, Tau, Rho,moneygrid,taugrid,mu_u) % Rotating movie
%% Input for estimating the COVARIANCE operator
[X,YY,YYdate]=phigrid(Date,Money,Tau,Rho);
%% Local linear COVARIANCE operator estimation with bandwidth h_phi_opt
h_phi_opt=[0.0780,0.0815,0.0780,0.0815];
phi_u_v=phi_est(moneygrid,taugrid,X,YY,h_phi_opt); % estimation (VERY LONG!!!!)
%% FPCA
[psi_u_v,expl_var,eig_sort,V_sort]=fpca(phi_u_v,moneygrid,taugrid,mu_u);
%% Plotting the covariance operator
subplot(1,2,1,'Position',[0.05 0.15 0.5 0.8])
PCOLOR(psi_u_v)
axis ij
axis square
caxis([min(min(psi_u_v)) max(max(psi_u_v))])
colorbar;
title(strcat('Covariance operator, h=[0.0239,0.0252,0.0239,0.0252]'))
subplot(1,2,2,'Position',[0.5 0.15 0.4 0.8])
plot (expl_var,'LineWidth',2)
xlim([1 100])
ylim([0.5 1])
ylabel('Variance explained')
xlabel('Index')
axis square
title(strcat('Cumulative proportion of variance explained by FPCs'))
%% Plot interepolated eigenfunctions
plotsurf_basis(V_sort,moneygrid,taugrid,dim)
%% Estimation of Zs + Residuals
K=2; % number of basis functions
DATA=[Date, Money,Tau,Rho];
Days=unique(Date);
DATA(DATA(:,2)>max(moneygrid)|DATA(:,2)<min(moneygrid),:)=[];
DATA(DATA(:,3)>max(taugrid)|DATA(:,3)<min(taugrid),:)=[];
[Z,RESID,EXPL_VARIANCE,A]=z_est(DATA,dim,K,V_sort,moneygrid,taugrid);